"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fastifyCaster = exports.caster = void 0;
const zod_1 = require("zod");
// optional, nullable 무관하게 ZodNumber 체크
function isZodNumberAnyway(zodType) {
    if (zodType instanceof zod_1.z.ZodNumber) {
        return true;
    }
    else if (zodType instanceof zod_1.z.ZodNullable &&
        zodType._def.innerType instanceof zod_1.z.ZodNumber) {
        return true;
    }
    else if (zodType instanceof zod_1.z.ZodOptional &&
        zodType._def.innerType instanceof zod_1.z.ZodNumber) {
    }
    else if (zodType instanceof zod_1.z.ZodOptional &&
        zodType._def.innerType instanceof zod_1.z.ZodOptional &&
        zodType._type.def.innerType instanceof zod_1.z.ZodNumber) {
        return true;
    }
    return false;
}
// ZodType을 이용해 raw를 Type Coercing
function caster(zodType, raw) {
    if (isZodNumberAnyway(zodType) && typeof raw === "string") {
        // number
        return Number(raw);
    }
    else if (zodType instanceof zod_1.z.ZodUnion &&
        zodType.options.some((opt) => isZodNumberAnyway(opt))) {
        // zArrayable Number 케이스 처리
        if (Array.isArray(raw)) {
            const numType = zodType.options.find((opt) => opt instanceof zod_1.z.ZodNumber);
            return raw.map((elem) => caster(numType, elem));
        }
        else {
            return Number(raw);
        }
    }
    else if (zodType instanceof zod_1.z.ZodBoolean &&
        (raw === "true" || raw === "false")) {
        // boolean
        return raw === "true";
    }
    else if (raw !== null &&
        Array.isArray(raw) &&
        zodType instanceof zod_1.z.ZodArray) {
        // array
        return raw.map((elem) => caster(zodType.element, elem));
    }
    else if (zodType instanceof zod_1.z.ZodObject &&
        typeof raw === "object" &&
        raw !== null) {
        // object
        return Object.keys(raw).reduce((r, rawKey) => {
            r[rawKey] = caster(zodType.shape[rawKey], raw[rawKey]);
            return r;
        }, {});
    }
    else if (zodType instanceof zod_1.z.ZodOptional) {
        // optional
        return caster(zodType._def.innerType, raw);
    }
    else if (zodType instanceof zod_1.z.ZodNullable) {
        // nullable
        return caster(zodType._def.innerType, raw);
    }
    else {
        // 나머지는 처리 안함
        return raw;
    }
}
exports.caster = caster;
function fastifyCaster(schema) {
    return zod_1.z.preprocess((raw) => {
        return caster(schema, raw);
    }, schema);
}
exports.fastifyCaster = fastifyCaster;
//# sourceMappingURL=caster.js.map