/// <reference types="node" />
import { FastifyInstance, FastifyReply, FastifyRequest } from "fastify";
import { IncomingMessage, Server, ServerResponse } from "http";
import { Context } from "./context";
import { ApiParam, ApiParamType } from "../types/types";
import { Syncer } from "../syncer/syncer";
import { SonamuDBConfig } from "../database/db";
import { ApiDecoratorOptions } from "./decorators";
export type SonamuConfig = {
    api: {
        dir: string;
    };
    sync: {
        targets: string[];
    };
    route: {
        prefix: string;
    };
};
type SonamuFastifyConfig = {
    contextProvider: (defaultContext: Pick<Context, "headers" | "reply">, request: FastifyRequest, reply: FastifyReply) => Context;
    guardHandler: (guard: string, request: FastifyRequest, api: {
        typeParameters: ApiParamType.TypeParam[];
        parameters: ApiParam[];
        returnType: ApiParamType;
        modelName: string;
        methodName: string;
        path: string;
        options: ApiDecoratorOptions;
    }) => void;
    cache?: {
        get: (key: string) => Promise<unknown | null>;
        put: (key: string, value: unknown, ttl?: number) => Promise<void>;
        resolveKey: (path: string, reqBody: {
            [key: string]: unknown;
        }) => {
            cache: false;
        } | {
            cache: true;
            key: string;
            ttl?: number;
        };
    };
};
declare class SonamuClass {
    isInitialized: boolean;
    private _apiRootPath;
    set apiRootPath(apiRootPath: string);
    get apiRootPath(): string;
    get appRootPath(): string;
    private _dbConfig;
    set dbConfig(dbConfig: SonamuDBConfig);
    get dbConfig(): SonamuDBConfig;
    private _syncer;
    set syncer(syncer: Syncer);
    get syncer(): Syncer;
    private _config;
    set config(config: SonamuConfig);
    get config(): SonamuConfig;
    init(doSilent?: boolean, enableSync?: boolean, apiRootPath?: string): Promise<void>;
    withFastify(server: FastifyInstance<Server, IncomingMessage, ServerResponse>, config: SonamuFastifyConfig): Promise<void>;
    destroy(): Promise<void>;
}
export declare const Sonamu: SonamuClass;
export {};
//# sourceMappingURL=sonamu.d.ts.map