"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.attachOnDuplicateUpdate = void 0;
const knex_1 = __importDefault(require("knex"));
function attachOnDuplicateUpdate() {
    knex_1.default.QueryBuilder.extend("onDuplicateUpdate", function (...columns) {
        if (columns.length === 0) {
            // 업데이트 할 컬럼이 없으면 onDuplicateUpdate 구문 처리 패스
            const { sql: originalSQL, bindings: originalBindings } = this.toSQL();
            return this.client.raw(originalSQL, originalBindings);
        }
        const { placeholders, bindings } = columns.reduce((result, column) => {
            if (typeof column === "string") {
                result.placeholders.push(`?? = Values(??)`);
                result.bindings.push(column, column);
            }
            else if (column && typeof column === "object") {
                Object.keys(column).forEach((key) => {
                    result.placeholders.push(`?? = ?`);
                    result.bindings.push(key, column[key]);
                });
            }
            else {
                throw new Error("onDuplicateUpdate error: expected column name to be string or object.");
            }
            return result;
        }, { placeholders: [], bindings: [] });
        const { sql: originalSQL, bindings: originalBindings } = this.toSQL();
        const newBindings = [...originalBindings, ...bindings];
        return this.client.raw(`${originalSQL} ON DUPLICATE KEY UPDATE ${placeholders.join(", ")}`, newBindings);
    });
}
exports.attachOnDuplicateUpdate = attachOnDuplicateUpdate;
//# sourceMappingURL=knex-on-duplicate-update.js.map