"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpsertBuilder = exports.isRefField = void 0;
const uuid_1 = require("uuid");
const lodash_1 = __importStar(require("lodash"));
const entity_manager_1 = require("../entity/entity-manager");
function isRefField(field) {
    return (field !== undefined &&
        field !== null &&
        field.of !== undefined &&
        field.uuid !== undefined);
}
exports.isRefField = isRefField;
class UpsertBuilder {
    constructor() {
        this.tables = new Map();
    }
    getTable(tableName) {
        var _a;
        const table = this.tables.get(tableName);
        if (table === undefined) {
            const tableSpec = (() => {
                try {
                    return entity_manager_1.EntityManager.getTableSpec(tableName);
                }
                catch (_a) {
                    return null;
                }
            })();
            this.tables.set(tableName, {
                references: new Set(),
                rows: [],
                uniqueColumns: (_a = tableSpec === null || tableSpec === void 0 ? void 0 : tableSpec.uniqueColumns) !== null && _a !== void 0 ? _a : [],
                uniquesMap: new Map(),
            });
        }
        return this.tables.get(tableName);
    }
    hasTable(tableName) {
        return this.tables.has(tableName);
    }
    register(tableName, row) {
        var _a;
        const table = this.getTable(tableName);
        // uuid 생성 로직
        let uuid;
        // 해당 테이블의 unique 컬럼들의 값을 통해 키 생성
        const uniqueKey = table.uniqueColumns
            .map((unqCol) => {
            var _a;
            const val = row[unqCol];
            if (isRefField(val)) {
                return val.uuid;
            }
            else {
                return (_a = row[unqCol]) !== null && _a !== void 0 ? _a : (0, uuid_1.v4)(); // nullable 컬럼은 uuid로 대체
            }
        })
            .join("---delimiter--");
        if (table.uniqueColumns.length > 0) {
            // 기존 키가 있는 경우 uuid 그대로 사용
            uuid = table.uniquesMap.get(uniqueKey);
        }
        // 없는 경우 uuid 생성하고, 생성한 uuid를 uniquesMap에 보관
        if (!uuid) {
            uuid = (0, uuid_1.v4)();
            table.uniquesMap.set(uniqueKey, uuid);
        }
        // 이 테이블에 사용된 RefField를 순회하여, 현재 테이블 정보에 어떤 필드를 참조하는지 추가
        // 이 정보를 나중에 치환할 때 사용
        row = Object.keys(row).reduce((r, rowKey) => {
            var _a;
            const rowValue = row[rowKey];
            if (isRefField(rowValue)) {
                (_a = rowValue.use) !== null && _a !== void 0 ? _a : (rowValue.use = "id");
                table.references.add(rowValue.of + "." + rowValue.use);
                r[rowKey] = rowValue;
            }
            else if (typeof rowValue === "object") {
                // object인 경우 JSON으로 변환
                r[rowKey] = rowValue === null ? null : JSON.stringify(rowValue);
            }
            else {
                r[rowKey] = rowValue;
            }
            return r;
        }, {});
        table.rows.push(Object.assign({ uuid }, row));
        return {
            of: tableName,
            uuid: (_a = row.uuid) !== null && _a !== void 0 ? _a : uuid,
        };
    }
    upsert(wdb, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.upsertOrInsert(wdb, tableName, "upsert");
        });
    }
    insertOnly(wdb, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.upsertOrInsert(wdb, tableName, "insert");
        });
    }
    upsertOrInsert(wdb, tableName, mode) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.hasTable(tableName) === false) {
                return [];
            }
            const table = this.tables.get(tableName);
            if (table === undefined) {
                throw new Error(`존재하지 않는 테이블 ${tableName}에 upsert 요청`);
            }
            else if (table.rows.length === 0) {
                throw new Error(`${tableName}에 upsert 할 데이터가 없습니다.`);
            }
            if (table.rows.some((row) => Object.entries(row).some(([, value]) => isRefField(value) && value.of !== tableName))) {
                throw new Error(`${tableName} 해결되지 않은 참조가 있습니다.`);
            }
            // 내부 참조 있는 경우 필터하여 분리
            const groups = (0, lodash_1.groupBy)(table.rows, (row) => Object.entries(row).some(([, value]) => isRefField(value))
                ? "selfRef"
                : "normal");
            const targetRows = groups.normal;
            // Insert On Duplicate Update
            const q = wdb.insert(targetRows).into(tableName);
            if (mode === "insert") {
                yield q;
            }
            else if (mode === "upsert") {
                yield q.onDuplicateUpdate.apply(q, Object.keys(targetRows[0]));
            }
            // 전체 테이블 순회하여 현재 테이블 참조하는 모든 테이블 추출
            const { references, refTables } = Array.from(this.tables).reduce((r, [, table]) => {
                const reference = Array.from(table.references.values()).find((ref) => ref.includes(tableName + "."));
                if (reference) {
                    r.references.push(reference);
                    r.refTables.push(table);
                }
                return r;
            }, {
                references: [],
                refTables: [],
            });
            const extractFields = lodash_1.default.uniq(references).map((reference) => reference.split(".")[1]);
            // UUID 기준으로 id 추출
            const uuids = table.rows.map((row) => row.uuid);
            const upsertedRows = yield wdb(tableName)
                .select(lodash_1.default.uniq(["uuid", "id", ...extractFields]))
                .whereIn("uuid", uuids);
            const uuidMap = new Map(upsertedRows.map((row) => [row.uuid, row]));
            // 해당 테이블 참조를 실제 밸류로 변경
            refTables.map((table) => {
                table.rows = table.rows.map((row) => {
                    Object.keys(row).map((key) => {
                        var _a;
                        const prop = row[key];
                        if (isRefField(prop) && prop.of === tableName) {
                            const parent = uuidMap.get(prop.uuid);
                            if (parent === undefined) {
                                console.error(prop);
                                throw new Error(`존재하지 않는 uuid ${prop.uuid} -- in ${tableName}`);
                            }
                            row[key] = parent[(_a = prop.use) !== null && _a !== void 0 ? _a : "id"];
                        }
                    });
                    return row;
                });
            });
            const ids = Array.from(uuidMap.values()).map((val) => val.id);
            if (groups.selfRef) {
                const selfRefIds = yield this.upsert(wdb, tableName);
                return [...ids, ...selfRefIds];
            }
            return ids;
        });
    }
    updateBatch(wdb, tableName, options) {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            options = (0, lodash_1.defaults)(options, {
                chunkSize: 500,
            });
            if (this.hasTable(tableName) === false) {
                return;
            }
            const table = this.tables.get(tableName);
            if (table.rows.length === 0) {
                return;
            }
            const chunks = (0, lodash_1.chunk)(table.rows, options.chunkSize);
            try {
                for (var _d = true, chunks_1 = __asyncValues(chunks), chunks_1_1; chunks_1_1 = yield chunks_1.next(), _a = chunks_1_1.done, !_a; _d = true) {
                    _c = chunks_1_1.value;
                    _d = false;
                    const chunk = _c;
                    yield Promise.all(chunk.map((_e) => __awaiter(this, void 0, void 0, function* () {
                        var { id } = _e, row = __rest(_e, ["id"]);
                        const { uuid } = row, update = __rest(row, ["uuid"]);
                        return yield wdb(tableName).where("id", id).update(update);
                    })));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = chunks_1.return)) yield _b.call(chunks_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
}
exports.UpsertBuilder = UpsertBuilder;
//# sourceMappingURL=upsert-builder.js.map