"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.i = exports.p = void 0;
const inflection_1 = require("inflection");
const model_1 = require("../utils/model");
exports.p = {
    integer,
    bigInteger,
    text,
    string,
    float,
    double,
    decimal,
    boolean,
    date,
    dateTime,
    time,
    timestamp,
    json,
    uuid,
    enums,
    virtual,
    relationOneToOne,
    relationBelongsToOne,
    relationHasMany,
    relationManyToMany,
};
function integer(name, option) {
    return Object.assign({ name, type: "integer" }, option);
}
function bigInteger(name, option) {
    return Object.assign({ name, type: "bigInteger" }, option);
}
function text(name, option) {
    return Object.assign({ name, type: "text" }, option);
}
function string(name, option) {
    return Object.assign({ name, type: "string" }, option);
}
function float(name, option) {
    return Object.assign({ name, type: "float", precision: 8, scale: 2 }, option);
}
function double(name, option) {
    return Object.assign({ name, type: "double", precision: 8, scale: 2 }, option);
}
function decimal(name, option) {
    return Object.assign({ name, type: "decimal", precision: 8, scale: 2 }, option);
}
function boolean(name, option) {
    return Object.assign({ name, type: "boolean" }, option);
}
function date(name, option) {
    if ((option === null || option === void 0 ? void 0 : option.now) === true) {
        delete option.now;
        option.dbDefault = "CURRENT_TIMESTAMP";
    }
    return Object.assign({ name, type: "date" }, option);
}
function dateTime(name, option) {
    if ((option === null || option === void 0 ? void 0 : option.now) === true) {
        delete option.now;
        option.dbDefault = "CURRENT_TIMESTAMP";
    }
    return Object.assign({ name, type: "dateTime" }, option);
}
function time(name, option) {
    if ((option === null || option === void 0 ? void 0 : option.now) === true) {
        delete option.now;
        option.dbDefault = "CURRENT_TIMESTAMP";
    }
    return Object.assign({ name, type: "time" }, option);
}
function timestamp(name, option) {
    if ((option === null || option === void 0 ? void 0 : option.now) === true) {
        delete option.now;
        option.dbDefault = "CURRENT_TIMESTAMP";
    }
    return Object.assign({ name, type: "timestamp" }, option);
}
function json(name, option) {
    return Object.assign({ name, type: "json" }, option);
}
function uuid(name, option) {
    return Object.assign({ name, type: "uuid" }, option);
}
function enums(name, option) {
    var _a;
    return Object.assign({ name, type: "enum", id: (_a = option.id) !== null && _a !== void 0 ? _a : `$Model${(0, inflection_1.camelize)(name)}` }, option);
}
function virtual(name, option) {
    return Object.assign({ name, type: "virtual" }, option);
}
function relationOneToOne(name, option) {
    return Object.assign({ name, type: "relation", relationType: "OneToOne" }, option);
}
function relationBelongsToOne(name, option) {
    return Object.assign({ name, type: "relation", relationType: "BelongsToOne" }, option);
}
function relationHasMany(name, option) {
    return Object.assign({ name, type: "relation", relationType: "HasMany" }, option);
}
function relationManyToMany(name, option) {
    return Object.assign({ name, type: "relation", relationType: "ManyToMany" }, option);
}
exports.i = {
    index,
    unique,
};
function index(columns) {
    return {
        type: "index",
        columns: (0, model_1.asArray)(columns),
    };
}
function unique(columns) {
    return {
        type: "unique",
        columns: (0, model_1.asArray)(columns),
    };
}
//# sourceMappingURL=entity-utils.js.map