import { EntityProp, RelationProp, SubsetQuery, EntityPropNode, EntityIndex, EntityJson, EntitySubsetRow } from "../types/types";
import { z } from "zod";
export declare class Entity {
    id: string;
    parentId?: string;
    table: string;
    title: string;
    names: {
        parentFs: string;
        fs: string;
        module: string;
    };
    props: EntityProp[];
    propsDict: {
        [key: string]: EntityProp;
    };
    relations: {
        [key: string]: RelationProp;
    };
    indexes: EntityIndex[];
    subsets: {
        [key: string]: string[];
    };
    types: {
        [name: string]: z.ZodTypeAny;
    };
    enums: {
        [enumId: string]: z.ZodEnum<any>;
    };
    enumLabels: {
        [enumId: string]: {
            [key: string]: string;
        };
    };
    constructor({ id, parentId, table, title, props, indexes, subsets, enums, }: EntityJson);
    getSubsetQuery(subsetKey: string): SubsetQuery;
    resolveSubsetQuery(prefix: string, fields: string[], isAlreadyOuterJoined?: boolean): SubsetQuery;
    fieldExprsToPropNodes(fieldExprs: string[], entity?: Entity): EntityPropNode[];
    getFieldExprs(prefix?: string, maxDepth?: number, froms?: string[]): string[];
    getTableColumns(): string[];
    registerModulePaths(): void;
    registerTableSpecs(): void;
    toJson(): EntityJson;
    save(): Promise<void>;
    getSubsetRows(_subsets?: {
        [key: string]: string[];
    }, prefixes?: string[]): EntitySubsetRow[];
    subsetRowsToSubsetFields(subsetRows: EntitySubsetRow[], subsetKey: string): string[];
    createProp(prop: EntityProp, at?: number): Promise<void>;
    analyzeSubsetField(subsetField: string): {
        entityId: string;
        propName: string;
    }[];
    modifyProp(newProp: EntityProp, at: number): Promise<void>;
    delProp(at: number): Promise<void>;
    getEntityIdFromSubsetField(subsetField: string): string;
    moveProp(at: number, to: number): Promise<void>;
}
//# sourceMappingURL=entity.d.ts.map