"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupErrorHandler = void 0;
const so_exceptions_1 = require("./so-exceptions");
function setupErrorHandler(server) {
    server.setErrorHandler((error, request, reply) => {
        var _a;
        (_a = error.statusCode) !== null && _a !== void 0 ? _a : (error.statusCode = 400);
        if ((0, so_exceptions_1.isSoException)(error) && error.payload && Array.isArray(error.payload)) {
            const issues = error.payload;
            const [issue] = issues;
            const message = `${issue.message} (${issue.path.join("/")})`;
            request.log.error(`${error.statusCode} ${message}`);
            reply.status(error.statusCode <= 501 ? error.statusCode : 501).send({
                name: error.name,
                code: error.code,
                message: message,
                validationErrors: error.validation,
                issues,
            });
        }
        else {
            request.log.error(`${error.statusCode} ${error.message}`);
            reply.status(error.statusCode <= 501 ? error.statusCode : 501).send({
                name: error.name,
                code: error.code,
                message: error.message,
                validationErrors: error.validation,
            });
        }
    });
}
exports.setupErrorHandler = setupErrorHandler;
//# sourceMappingURL=error-handler.js.map