"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TargetNotFoundException = exports.DuplicateRowException = exports.AlreadyProcessedException = exports.InternalServerErrorException = exports.ServiceUnavailableException = exports.NotFoundException = exports.UnauthorizedException = exports.BadRequestException = exports.isSoException = exports.SoException = void 0;
class SoException extends Error {
    constructor(statusCode, message, payload) {
        super(message);
        this.statusCode = statusCode;
        this.message = message;
        this.payload = payload;
    }
}
exports.SoException = SoException;
function isSoException(err) {
    return err.statusCode !== undefined;
}
exports.isSoException = isSoException;
/*
    잘못된 매개변수 등 요청사항에 문제가 있는 경우
*/
class BadRequestException extends SoException {
    constructor(message = "Bad Request", payload) {
        super(400, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.BadRequestException = BadRequestException;
/*
    로그인이 반드시 필요한 케이스에 로그아웃 상태인 경우 / 접근 권한이 없는 요청시
*/
class UnauthorizedException extends SoException {
    constructor(message = "Unauthorized", payload) {
        super(401, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.UnauthorizedException = UnauthorizedException;
/*
    존재하지 않는 레코드에 접근시
*/
class NotFoundException extends SoException {
    constructor(message = "Not Found", payload) {
        super(404, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.NotFoundException = NotFoundException;
/*
    현재 상태에서 처리가 불가능한 케이스
*/
class ServiceUnavailableException extends SoException {
    constructor(message = "Service Unavailable", payload) {
        super(503, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.ServiceUnavailableException = ServiceUnavailableException;
/*
    내부 처리 로직 (외부 API 콜 포함) 오류 발생시
*/
class InternalServerErrorException extends SoException {
    constructor(message = "Internal Server Error", payload) {
        super(500, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.InternalServerErrorException = InternalServerErrorException;
/*
    이미 처리함
*/
class AlreadyProcessedException extends SoException {
    constructor(message = "Already Processed", payload) {
        super(641, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.AlreadyProcessedException = AlreadyProcessedException;
/*
    중복 허용하지 않는 케이스에 중복 요청
*/
class DuplicateRowException extends SoException {
    constructor(message = "Duplicate Row", payload) {
        super(642, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.DuplicateRowException = DuplicateRowException;
/*
    뭔가를 하려고 했으나 대상이 없음
*/
class TargetNotFoundException extends SoException {
    constructor(message = "Target Not Found", payload) {
        super(620, message, payload);
        this.message = message;
        this.payload = payload;
    }
}
exports.TargetNotFoundException = TargetNotFoundException;
//# sourceMappingURL=so-exceptions.js.map