"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityManager = void 0;
const chalk_1 = __importDefault(require("chalk"));
const glob_1 = require("glob");
const inflection_1 = require("inflection");
const path_1 = __importDefault(require("path"));
const entity_1 = require("./entity");
const sonamu_1 = require("../api/sonamu");
const fs_1 = require("fs");
class EntityManagerClass {
    constructor() {
        this.entities = new Map();
        this.modulePaths = new Map();
        this.tableSpecs = new Map();
        this.isAutoloaded = false;
    }
    // 경로 전달받아 모든 SMD 파일 로드
    autoload(doSilent = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isAutoloaded) {
                return;
            }
            const pathPattern = path_1.default.join(sonamu_1.Sonamu.apiRootPath, "/src/application/**/*.entity.json");
            !doSilent && console.log(chalk_1.default.yellow(`autoload ${pathPattern}`));
            return new Promise((resolve) => {
                (0, glob_1.glob)(path_1.default.resolve(pathPattern), (_err, files) => {
                    Promise.all(files.map((file) => __awaiter(this, void 0, void 0, function* () {
                        this.register(JSON.parse((0, fs_1.readFileSync)(file).toString()));
                    }))).then(() => {
                        resolve("ok");
                        this.isAutoloaded = true;
                    });
                });
            });
        });
    }
    register(smdInput) {
        const entity = new entity_1.Entity(smdInput);
        this.entities.set(smdInput.id, entity);
    }
    get(entityId) {
        const entity = this.entities.get(entityId);
        if (entity === undefined) {
            throw new Error(`존재하지 않는 SMD 요청 ${entityId}`);
        }
        return entity;
    }
    exists(entityId) {
        const entity = this.entities.get(entityId);
        return entity !== undefined;
    }
    getAllIds() {
        return Array.from(exports.EntityManager.entities.keys());
    }
    getAllParentIds() {
        return this.getAllIds().filter((entityId) => {
            const entity = this.get(entityId);
            return entity.parentId === undefined;
        });
    }
    getChildrenIds(parentId) {
        return this.getAllIds().filter((entityId) => {
            const entity = this.get(entityId);
            return entity.parentId === parentId;
        });
    }
    setModulePath(key, modulePath) {
        // console.debug(chalk.cyan(`setModulePath :: ${key} :: ${modulePath}`));
        this.modulePaths.set(key, modulePath);
    }
    getModulePath(key) {
        const modulePath = this.modulePaths.get(key);
        if (modulePath === undefined) {
            throw new Error(`존재하지 않는 모듈 패스 요청 ${key}`);
        }
        return modulePath;
    }
    setTableSpec(tableSpec) {
        this.tableSpecs.set(tableSpec.name, tableSpec);
    }
    getTableSpec(key) {
        const tableSpec = this.tableSpecs.get(key);
        if (tableSpec === undefined) {
            throw new Error(`존재하지 않는 테이블 스펙 요청 ${key}`);
        }
        return tableSpec;
    }
    getNamesFromId(entityId) {
        // entityId가 단복수 동형 단어인 경우 List 붙여서 생성
        const pluralized = (0, inflection_1.pluralize)(entityId) === entityId
            ? `${entityId}List`
            : (0, inflection_1.pluralize)(entityId);
        return {
            fs: (0, inflection_1.dasherize)((0, inflection_1.underscore)(entityId)).toLowerCase(),
            fsPlural: (0, inflection_1.dasherize)((0, inflection_1.underscore)(pluralized)).toLowerCase(),
            camel: (0, inflection_1.camelize)(entityId, true),
            camelPlural: (0, inflection_1.camelize)(pluralized, true),
            capital: entityId,
            capitalPlural: pluralized,
            upper: entityId.toUpperCase(),
            constant: (0, inflection_1.underscore)(entityId).toUpperCase(),
        };
    }
}
exports.EntityManager = new EntityManagerClass();
//# sourceMappingURL=entity-manager.js.map