"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Entity = void 0;
const lodash_1 = __importStar(require("lodash"));
const entity_manager_1 = require("../entity/entity-manager");
const inflection_1 = require("inflection");
const types_1 = require("../types/types");
const inflection_2 = __importDefault(require("inflection"));
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const sonamu_1 = require("../api/sonamu");
class Entity {
    constructor({ id, parentId, table, title, props, indexes, subsets, }) {
        this.types = {};
        this.enums = {};
        this.enumLabels = {};
        // id
        this.id = id;
        this.parentId = parentId;
        this.title = title !== null && title !== void 0 ? title : this.id;
        this.table = table !== null && table !== void 0 ? table : (0, inflection_1.underscore)((0, inflection_1.pluralize)(id));
        // props
        if (props) {
            this.props = props.map((prop) => {
                if ((0, types_1.isEnumProp)(prop)) {
                    if (prop.id.includes("$Model")) {
                        prop.id = prop.id.replace("$Model", id);
                    }
                }
                return prop;
            });
            this.propsDict = props.reduce((result, prop) => {
                return Object.assign(Object.assign({}, result), { [prop.name]: prop });
            }, {});
            // relations
            this.relations = props
                .filter((prop) => (0, types_1.isRelationProp)(prop))
                .reduce((result, prop) => {
                return Object.assign(Object.assign({}, result), { [prop.name]: prop });
            }, {});
        }
        else {
            this.props = [];
            this.propsDict = {};
            this.relations = {};
        }
        // indexes
        this.indexes = indexes !== null && indexes !== void 0 ? indexes : [];
        // subsets
        this.subsets = subsets !== null && subsets !== void 0 ? subsets : {};
        // names
        this.names = {
            fs: parentId === undefined
                ? (0, inflection_1.dasherize)((0, inflection_1.underscore)(id)).toLowerCase()
                : (0, inflection_1.dasherize)(parentId).toLowerCase(),
            module: id,
        };
        this.registerModulePaths();
        this.registerTableSpecs();
    }
    /*
      subset SELECT/JOIN/LOADER 결과 리턴
    */
    getSubsetQuery(subsetKey) {
        const subset = this.subsets[subsetKey];
        const result = this.resolveSubsetQuery("", subset);
        return result;
    }
    /*
     */
    resolveSubsetQuery(prefix, fields, isAlreadyOuterJoined = false) {
        // prefix 치환 (prefix는 ToOneRelation이 복수로 붙은 경우 모두 __로 변경됨)
        prefix = prefix.replace(/\./g, "__");
        // 서브셋을 1뎁스만 분리하여 그룹핑
        const subsetGroup = lodash_1.default.groupBy(fields, (field) => {
            if (field.includes(".")) {
                const [rel] = field.split(".");
                return rel;
            }
            else {
                return "";
            }
        });
        const result = Object.keys(subsetGroup).reduce((r, groupKey) => {
            const fields = subsetGroup[groupKey];
            // 현재 테이블 필드셋은 select, virtual에 추가하고 리턴
            if (groupKey === "") {
                const realFields = fields.filter((field) => !(0, types_1.isVirtualProp)(this.propsDict[field]));
                const virtualFields = fields.filter((field) => (0, types_1.isVirtualProp)(this.propsDict[field]));
                if (prefix === "") {
                    // 현재 테이블인 경우
                    r.select = r.select.concat(realFields.map((field) => `${this.table}.${field}`));
                    r.virtual = r.virtual.concat(virtualFields);
                }
                else {
                    // 넘어온 테이블인 경우
                    r.select = r.select.concat(realFields.map((field) => `${prefix}.${field} as ${prefix}__${field}`));
                }
                return r;
            }
            const relation = this.relations[groupKey];
            if (relation === undefined) {
                throw new Error(`존재하지 않는 relation 참조 ${groupKey}`);
            }
            const relSMD = entity_manager_1.EntityManager.get(relation.with);
            if ((0, types_1.isOneToOneRelationProp)(relation) ||
                (0, types_1.isBelongsToOneRelationProp)(relation)) {
                // -One Relation: JOIN 으로 처리
                const relFields = fields.map((field) => field.split(".").slice(1).join("."));
                // -One Relation에서 id 필드만 참조하는 경우 릴레이션 넘기지 않고 리턴
                if (relFields.length === 1 && relFields[0] === "id") {
                    if (prefix === "") {
                        r.select = r.select.concat(`${this.table}.${groupKey}_id`);
                    }
                    else {
                        r.select = r.select.concat(`${prefix}.${groupKey}_id as ${prefix}__${groupKey}_id`);
                    }
                    return r;
                }
                // innerOrOuter
                const innerOrOuter = (() => {
                    var _a;
                    if (isAlreadyOuterJoined) {
                        return "outer";
                    }
                    if ((0, types_1.isOneToOneRelationProp)(relation)) {
                        if (relation.hasJoinColumn === true &&
                            ((_a = relation.nullable) !== null && _a !== void 0 ? _a : false) === false) {
                            return "inner";
                        }
                        else {
                            return "outer";
                        }
                    }
                    else {
                        if (relation.nullable) {
                            return "outer";
                        }
                        else {
                            return "inner";
                        }
                    }
                })();
                const relSubsetQuery = relSMD.resolveSubsetQuery(`${prefix !== "" ? prefix + "." : ""}${groupKey}`, relFields, innerOrOuter === "outer");
                r.select = r.select.concat(relSubsetQuery.select);
                r.virtual = r.virtual.concat(relSubsetQuery.virtual);
                const joinAs = prefix === "" ? groupKey : prefix + "__" + groupKey;
                const fromTable = prefix === "" ? this.table : prefix;
                let joinClause;
                if (relation.customJoinClause) {
                    joinClause = {
                        custom: relation.customJoinClause,
                    };
                }
                else {
                    let from, to;
                    if ((0, types_1.isOneToOneRelationProp)(relation)) {
                        if (relation.hasJoinColumn) {
                            from = `${fromTable}.${relation.name}_id`;
                            to = `${joinAs}.id`;
                        }
                        else {
                            from = `${fromTable}.id`;
                            to = `${joinAs}.${(0, inflection_1.underscore)(this.names.fs.replace(/\-/g, "_"))}_id`;
                        }
                    }
                    else {
                        from = `${fromTable}.${relation.name}_id`;
                        to = `${joinAs}.id`;
                    }
                    joinClause = {
                        from,
                        to,
                    };
                }
                r.joins.push(Object.assign({ as: joinAs, join: innerOrOuter, table: relSMD.table }, joinClause));
                // BelongsToOne 밑에 HasMany가 붙은 경우
                if (relSubsetQuery.loaders.length > 0) {
                    const convertedLoaders = relSubsetQuery.loaders.map((loader) => {
                        const newAs = [groupKey, loader.as].join("__");
                        return {
                            as: newAs,
                            table: loader.table,
                            manyJoin: loader.manyJoin,
                            oneJoins: loader.oneJoins,
                            select: loader.select,
                        };
                    });
                    r.loaders = [...r.loaders, ...convertedLoaders];
                }
                r.joins = r.joins.concat(relSubsetQuery.joins);
            }
            else if ((0, types_1.isHasManyRelationProp)(relation) ||
                (0, types_1.isManyToManyRelationProp)(relation)) {
                // -Many Relation: Loader 로 처리
                const relFields = fields.map((field) => field.split(".").slice(1).join("."));
                const relSubsetQuery = relSMD.resolveSubsetQuery("", relFields);
                let manyJoin;
                if ((0, types_1.isHasManyRelationProp)(relation)) {
                    manyJoin = {
                        fromTable: this.table,
                        fromCol: "id",
                        idField: prefix === "" ? `id` : `${prefix}__id`,
                        toTable: relSMD.table,
                        toCol: relation.joinColumn,
                    };
                }
                else if ((0, types_1.isManyToManyRelationProp)(relation)) {
                    const [table1, table2] = relation.joinTable.split("__");
                    manyJoin = {
                        fromTable: this.table,
                        fromCol: "id",
                        idField: prefix === "" ? `id` : `${prefix}__id`,
                        through: {
                            table: relation.joinTable,
                            fromCol: `${inflection_2.default.singularize(table1)}_id`,
                            toCol: `${inflection_2.default.singularize(table2)}_id`,
                        },
                        toTable: relSMD.table,
                        toCol: "id",
                    };
                }
                else {
                    throw new Error();
                }
                r.loaders.push({
                    as: groupKey,
                    table: relSMD.table,
                    manyJoin,
                    oneJoins: relSubsetQuery.joins,
                    select: relSubsetQuery.select,
                    loaders: relSubsetQuery.loaders,
                });
            }
            return r;
        }, {
            select: [],
            virtual: [],
            joins: [],
            loaders: [],
        });
        return result;
    }
    /*
      FieldExpr[] 을 SMDPropNode[] 로 변환
    */
    fieldExprsToPropNodes(fieldExprs, smd = this) {
        const groups = fieldExprs.reduce((result, fieldExpr) => {
            var _a;
            let key, value, elseExpr;
            if (fieldExpr.includes(".")) {
                [key, ...elseExpr] = fieldExpr.split(".");
                value = elseExpr.join(".");
            }
            else {
                key = "";
                value = fieldExpr;
            }
            result[key] = ((_a = result[key]) !== null && _a !== void 0 ? _a : []).concat(value);
            return result;
        }, {});
        return Object.keys(groups)
            .map((key) => {
            const group = groups[key];
            // 일반 prop 처리
            if (key === "") {
                return group.map((propName) => {
                    // uuid 개별 처리
                    if (propName === "uuid") {
                        return {
                            nodeType: "plain",
                            prop: {
                                type: "string",
                                name: "uuid",
                                length: 128,
                            },
                            children: [],
                        };
                    }
                    const prop = smd.propsDict[propName];
                    if (prop === undefined) {
                        throw new Error(`${this.id} -- 잘못된 FieldExpr ${propName}`);
                    }
                    return {
                        nodeType: "plain",
                        prop,
                        children: [],
                    };
                });
            }
            // relation prop 처리
            const prop = smd.propsDict[key];
            if (!(0, types_1.isRelationProp)(prop)) {
                throw new Error(`잘못된 FieldExpr ${key}.${group[0]}`);
            }
            const relSMD = entity_manager_1.EntityManager.get(prop.with);
            // relation -One 에 id 필드 하나인 경우
            if ((0, types_1.isBelongsToOneRelationProp)(prop) || (0, types_1.isOneToOneRelationProp)(prop)) {
                if (group.length == 1 && (group[0] === "id" || group[0] == "id?")) {
                    // id 하나만 있는지 체크해서, 하나만 있으면 상위 prop으로 id를 리턴
                    const idProp = relSMD.propsDict.id;
                    return {
                        nodeType: "plain",
                        prop: Object.assign(Object.assign({}, idProp), { name: key + "_id", nullable: prop.nullable }),
                        children: [],
                    };
                }
            }
            // -One 그외의 경우 object로 리턴
            // -Many의 경우 array로 리턴
            // Recursive 로 뎁스 처리
            const children = this.fieldExprsToPropNodes(group, relSMD);
            const nodeType = (0, types_1.isBelongsToOneRelationProp)(prop) || (0, types_1.isOneToOneRelationProp)(prop)
                ? "object"
                : "array";
            return {
                prop,
                children,
                nodeType,
            };
        })
            .flat();
    }
    getFieldExprs(prefix = "", maxDepth = 3, froms = []) {
        return this.props
            .map((prop) => {
            const propName = [prefix, prop.name].filter((v) => v !== "").join(".");
            if (propName === prefix) {
                return null;
            }
            if ((0, types_1.isRelationProp)(prop)) {
                if (maxDepth < 0) {
                    return null;
                }
                if (froms.includes(prop.with)) {
                    // 역방향 relation인 경우 제외
                    return null;
                }
                // 정방향 relation인 경우 recursive 콜
                const relMd = entity_manager_1.EntityManager.get(prop.with);
                return relMd.getFieldExprs(propName, maxDepth - 1, [
                    ...froms,
                    this.id,
                ]);
            }
            return propName;
        })
            .flat()
            .filter((f) => f !== null);
    }
    registerModulePaths() {
        const basePath = `${this.names.fs}`;
        // base-scheme
        entity_manager_1.EntityManager.setModulePath(`${this.id}BaseSchema`, `${basePath}/${this.names.fs}.generated`);
        // subset
        if (Object.keys(this.subsets).length > 0) {
            entity_manager_1.EntityManager.setModulePath(`${this.id}SubsetKey`, `${basePath}/${this.names.fs}.generated`);
            entity_manager_1.EntityManager.setModulePath(`${this.id}SubsetMapping`, `${basePath}/${this.names.fs}.generated`);
            Object.keys(this.subsets).map((subsetKey) => {
                entity_manager_1.EntityManager.setModulePath(`${this.id}Subset${subsetKey.toUpperCase()}`, `${basePath}/${this.names.fs}.generated`);
            });
        }
        // types
        const typesModulePath = `${basePath}/${this.names.fs}.types`;
        const typesFileDistPath = path_1.default.join(sonamu_1.Sonamu.apiRootPath, `dist/application/${typesModulePath}.js`);
        if ((0, fs_1.existsSync)(typesFileDistPath)) {
            const importPath = path_1.default.relative(__dirname, typesFileDistPath);
            Promise.resolve().then(() => __importStar(require(importPath))).then((t) => {
                this.types = Object.keys(t).reduce((result, key) => {
                    entity_manager_1.EntityManager.setModulePath(key, typesModulePath);
                    return Object.assign(Object.assign({}, result), { [key]: t[key] });
                }, {});
            });
        }
        // enums
        const enumsModulePath = `${basePath}/${this.names.fs}.generated`;
        const enumsFileDistPath = path_1.default.join(sonamu_1.Sonamu.apiRootPath, `/dist/application/${enumsModulePath}.js`);
        if ((0, fs_1.existsSync)(enumsFileDistPath)) {
            const importPath = path_1.default.relative(__dirname, enumsFileDistPath);
            Promise.resolve().then(() => __importStar(require(importPath))).then((t) => {
                this.enums = Object.keys(t).reduce((result, key) => {
                    entity_manager_1.EntityManager.setModulePath(key, enumsModulePath);
                    // Enum Labels 별도 처리
                    if (key === (0, inflection_1.underscore)(this.id).toUpperCase()) {
                        this.enumLabels = t[key];
                    }
                    return Object.assign(Object.assign({}, result), { [key]: t[key] });
                }, {});
            });
        }
    }
    registerTableSpecs() {
        const uniqueColumns = (0, lodash_1.uniq)(this.indexes
            .filter((idx) => idx.type === "unique")
            .map((idx) => idx.columns)
            .flat());
        entity_manager_1.EntityManager.setTableSpec({
            name: this.table,
            uniqueColumns,
        });
    }
}
exports.Entity = Entity;
//# sourceMappingURL=entity.js.map