import { Knex } from "knex";
import { GenMigrationCode, MigrationColumn, MigrationForeign, MigrationIndex, MigrationSet, MigrationSetAndJoinTable } from "../types/types";
import { Entity } from "./entity";
declare type MigratorMode = "dev" | "deploy";
export declare type MigratorOptions = {
    readonly mode: MigratorMode;
};
export declare class Migrator {
    readonly mode: MigratorMode;
    targets: {
        compare?: Knex;
        pending: Knex;
        shadow: Knex;
        apply: Knex[];
    };
    constructor(options: MigratorOptions);
    clearPendingList(): Promise<void>;
    check(): Promise<void>;
    run(): Promise<void>;
    rollback(): Promise<void>;
    cleanUpDist(force?: boolean): Promise<void>;
    runShadowTest(): Promise<boolean>;
    resetAll(): Promise<void>;
    compareMigrations(): Promise<GenMigrationCode[]>;
    getMigrationSetFromDB(table: string): Promise<MigrationSet | null>;
    resolveDBColType(colType: string, colField: string): Pick<MigrationColumn, "type" | "unsigned" | "length" | "precision" | "scale">;
    readTable(tableName: string): Promise<[DBColumn[], DBIndex[], DBForeign[]]>;
    getMigrationSetFromMD(smd: Entity): MigrationSetAndJoinTable;
    genColumnDefinitions(columns: MigrationColumn[]): string[];
    genIndexDefinitions(indexes: MigrationIndex[]): string[];
    genForeignDefinitions(table: string, foreigns: MigrationForeign[]): {
        up: string[];
        down: string[];
    };
    generateCreateCode_ColumnAndIndexes(table: string, columns: MigrationColumn[], indexes: MigrationIndex[]): GenMigrationCode;
    generateCreateCode_Foreign(table: string, foreigns: MigrationForeign[]): GenMigrationCode[];
    showMigrationSet(which: string, migrationSet: MigrationSet): void;
    generateAlterCode_ColumnAndIndexes(table: string, smdColumns: MigrationColumn[], smdIndexes: MigrationIndex[], dbColumns: MigrationColumn[], dbIndexes: MigrationIndex[]): GenMigrationCode[];
    getAlterColumnsTo(smdColumns: MigrationColumn[], dbColumns: MigrationColumn[]): {
        add: MigrationColumn[];
        drop: MigrationColumn[];
        alter: MigrationColumn[];
    };
    getAlterColumnLinesTo(columnsTo: ReturnType<Migrator["getAlterColumnsTo"]>, smdColumns: MigrationColumn[]): {
        add: {
            up: string[];
            down: string[];
        };
        drop: {
            up: string[];
            down: string[];
        };
        alter: {
            up: string[];
            down: string[];
        };
    };
    getAlterIndexesTo(smdIndexes: MigrationIndex[], dbIndexes: MigrationIndex[]): {
        add: MigrationIndex[];
        drop: MigrationIndex[];
    };
    getAlterIndexLinesTo(indexesTo: ReturnType<Migrator["getAlterIndexesTo"]>, columnsTo: ReturnType<Migrator["getAlterColumnsTo"]>): {
        add: {
            up: string[];
            down: string[];
        };
        drop: {
            up: string[];
            down: string[];
        };
    };
    generateAlterCode_Foreigns(table: string, smdForeigns: MigrationForeign[], dbForeigns: MigrationForeign[]): GenMigrationCode[];
    destroy(): Promise<void>;
}
declare type DBColumn = {
    Field: string;
    Type: string;
    Null: string;
    Key: string;
    Default: string | null;
    Extra: string;
};
declare type DBIndex = {
    Table: string;
    Non_unique: number;
    Key_name: string;
    Seq_in_index: number;
    Column_name: string;
    Collation: string | null;
    Cardinality: number | null;
    Sub_part: number | null;
    Packed: string | null;
    Null: string;
    Index_type: string;
    Comment: string;
    Index_comment: string;
    Visible: string;
    Expression: string | null;
};
declare type DBForeign = {
    keyName: string;
    from: string;
    referencesTable: string;
    referencesField: string;
    onDelete: string;
    onUpdate: string;
};
export {};
//# sourceMappingURL=migrator.d.ts.map