"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SMDManager = void 0;
const chalk_1 = __importDefault(require("chalk"));
const glob_1 = require("glob");
const inflection_1 = require("inflection");
const path_1 = __importDefault(require("path"));
const smd_1 = require("./smd");
const sonamu_1 = require("../api/sonamu");
class SMDManagerClass {
    constructor() {
        this.SMDs = new Map();
        this.modulePaths = new Map();
        this.tableSpecs = new Map();
        this.isAutoloaded = false;
    }
    // 경로 전달받아 모든 SMD 파일 로드
    autoload(doSilent = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isAutoloaded) {
                return;
            }
            const pathPattern = path_1.default.join(sonamu_1.Sonamu.apiRootPath, "/dist/application/**/*.smd.js");
            !doSilent && console.log(chalk_1.default.yellow(`autoload ${pathPattern}`));
            return new Promise((resolve) => {
                (0, glob_1.glob)(path_1.default.resolve(pathPattern), (_err, files) => {
                    const importPaths = files.map((filePath) => path_1.default.relative(__dirname, filePath));
                    Promise.all(importPaths.map((importPath) => __awaiter(this, void 0, void 0, function* () {
                        const imported = yield Promise.resolve(`${importPath}`).then(s => __importStar(require(s)));
                        Object.values(imported).map((smdInput) => this.register(smdInput));
                        return imported;
                    }))).then(() => {
                        resolve("ok");
                        this.isAutoloaded = true;
                    });
                });
            });
        });
    }
    register(smdInput) {
        const smd = new smd_1.SMD(smdInput);
        this.SMDs.set(smdInput.id, smd);
    }
    get(smdId) {
        const smd = this.SMDs.get(smdId);
        if (smd === undefined) {
            throw new Error(`존재하지 않는 SMD 요청 ${smdId}`);
        }
        return smd;
    }
    exists(smdId) {
        const smd = this.SMDs.get(smdId);
        return smd !== undefined;
    }
    getAllIds() {
        return Array.from(exports.SMDManager.SMDs.keys());
    }
    getAllParentIds() {
        return this.getAllIds().filter((smdId) => {
            const smd = this.get(smdId);
            return smd.parentId === undefined;
        });
    }
    getChildrenIds(parentId) {
        return this.getAllIds().filter((smdId) => {
            const smd = this.get(smdId);
            return smd.parentId === parentId;
        });
    }
    setModulePath(key, modulePath) {
        // console.debug(chalk.cyan(`setModulePath :: ${key} :: ${modulePath}`));
        this.modulePaths.set(key, modulePath);
    }
    getModulePath(key) {
        const modulePath = this.modulePaths.get(key);
        if (modulePath === undefined) {
            throw new Error(`존재하지 않는 모듈 패스 요청 ${key}`);
        }
        return modulePath;
    }
    setTableSpec(tableSpec) {
        this.tableSpecs.set(tableSpec.name, tableSpec);
    }
    getTableSpec(key) {
        const tableSpec = this.tableSpecs.get(key);
        if (tableSpec === undefined) {
            throw new Error(`존재하지 않는 테이블 스펙 요청 ${key}`);
        }
        return tableSpec;
    }
    getNamesFromId(smdId) {
        // entityId가 단복수 동형 단어인 경우 List 붙여서 생성
        const pluralized = (0, inflection_1.pluralize)(smdId) === smdId ? `${smdId}List` : (0, inflection_1.pluralize)(smdId);
        return {
            fs: (0, inflection_1.dasherize)((0, inflection_1.underscore)(smdId)).toLowerCase(),
            fsPlural: (0, inflection_1.dasherize)((0, inflection_1.underscore)(pluralized)).toLowerCase(),
            camel: (0, inflection_1.camelize)(smdId, true),
            camelPlural: (0, inflection_1.camelize)(pluralized, true),
            capital: smdId,
            capitalPlural: pluralized,
            upper: smdId.toUpperCase(),
            constant: (0, inflection_1.underscore)(smdId).toUpperCase(),
        };
    }
}
exports.SMDManager = new SMDManagerClass();
//# sourceMappingURL=smd-manager.js.map