import * as ts from "typescript";
import { ApiParam, ApiParamType, EntityProp, EntityPropNode } from "../types/types";
import { ApiDecoratorOptions } from "../api/decorators";
import { z } from "zod";
import { TemplateKey, PathAndCode, TemplateOptions, GenerateOptions, RenderingNode } from "../types/types";
import { Template } from "../templates/base-template";
type PathAndChecksum = {
    path: string;
    checksum: string;
};
export type RenderedTemplate = {
    target: string;
    path: string;
    body: string;
    importKeys: string[];
    customHeaders?: string[];
    preTemplates?: {
        key: TemplateKey;
        options: TemplateOptions[TemplateKey];
    }[];
};
export declare class Syncer {
    apis: {
        typeParameters: ApiParamType.TypeParam[];
        parameters: ApiParam[];
        returnType: ApiParamType;
        modelName: string;
        methodName: string;
        path: string;
        options: ApiDecoratorOptions;
    }[];
    types: {
        [typeName: string]: z.ZodObject<any>;
    };
    models: {
        [modelName: string]: unknown;
    };
    get checksumsPath(): string;
    constructor();
    sync(): Promise<void>;
    getEntityIdFromPath(filePaths: string[]): string[];
    actionGenerateSchemas(): Promise<string[]>;
    actionGenerateServices(entityIds: string[]): Promise<string[]>;
    actionGenerateHttps(entityIds: string[]): Promise<string[]>;
    copyFileWithReplaceCoreToShared(fromPath: string, toPath: string): Promise<void>;
    actionSyncFilesToTargets(tsPaths: string[]): Promise<string[]>;
    getCurrentChecksums(): Promise<PathAndChecksum[]>;
    getPreviousChecksums(): Promise<PathAndChecksum[]>;
    saveChecksums(checksums: PathAndChecksum[]): Promise<void>;
    getChecksumOfFile(filePath: string): Promise<string>;
    readApisFromFile(filePath: string): Promise<{
        typeParameters: ApiParamType.TypeParam[];
        parameters: ApiParam[];
        returnType: ApiParamType;
        modelName: string;
        methodName: string;
        path: string;
        options: ApiDecoratorOptions;
    }[]>;
    resolveTypeNode(typeNode: ts.TypeNode): ApiParamType;
    resolveParamDec: (paramDec: {
        name: ts.BindingName;
        type: ts.TypeNode;
        optional?: boolean;
        defaultDef?: string;
    }, index?: number) => ApiParam;
    printNode(node: ts.Node | undefined, sourceFile: ts.SourceFile): string | undefined;
    autoloadApis(): Promise<{
        typeParameters: ApiParamType.TypeParam[];
        parameters: ApiParam[];
        returnType: ApiParamType;
        modelName: string;
        methodName: string;
        path: string;
        options: ApiDecoratorOptions;
    }[]>;
    autoloadModels(): Promise<{
        [modelName: string]: unknown;
    }>;
    autoloadTypes(doRefresh?: boolean): Promise<{
        [typeName: string]: z.ZodObject<any>;
    }>;
    getTemplate(key: TemplateKey): Template;
    renderTemplate<T extends keyof TemplateOptions>(key: T, options: TemplateOptions[T]): Promise<PathAndCode[]>;
    resolveRenderedTemplate(key: TemplateKey, result: RenderedTemplate): Promise<PathAndCode>;
    writeCodeToPath(pathAndCode: PathAndCode): Promise<string[]>;
    generateTemplate(key: TemplateKey, templateOptions: any, _generateOptions?: GenerateOptions): Promise<string[][]>;
    checkExistsGenCode(entityId: string, templateKey: TemplateKey, enumId?: string): {
        subPath: string;
        fullPath: string;
        isExists: boolean;
    };
    checkExists(entityId: string, enums: {
        [name: string]: z.ZodEnum<any>;
    }): Record<`${TemplateKey}${string}`, boolean>;
    getZodTypeById(zodTypeId: string): Promise<z.ZodTypeAny>;
    propNodeToZodType(propNode: EntityPropNode): Promise<z.ZodTypeAny>;
    propToZodType(prop: EntityProp): Promise<z.ZodTypeAny>;
    resolveRenderType(key: string, zodType: z.ZodTypeAny): RenderingNode["renderType"];
    zodTypeToRenderingNode(zodType: z.ZodTypeAny, baseKey?: string): RenderingNode;
    getColumnsNode(entityId: string, subsetKey: string): Promise<RenderingNode>;
    createEntity(entityId: string, parentId?: string, table?: string, title?: string): Promise<void>;
    delEntity(entityId: string): Promise<{
        delPaths: string[];
    }>;
}
export {};
//# sourceMappingURL=syncer.d.ts.map