"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLabel = exports.Template__view_enums_dropdown = void 0;
const entity_manager_1 = require("../entity/entity-manager");
const base_template_1 = require("./base-template");
const inflection_1 = require("inflection");
class Template__view_enums_dropdown extends base_template_1.Template {
    constructor() {
        super("view_enums_dropdown");
    }
    getTargetAndPath(names, enumId) {
        return {
            target: "web/src/components",
            path: `${names.fs}/${enumId}Dropdown.tsx`,
        };
    }
    render({ entityId, enumId }) {
        const names = entity_manager_1.EntityManager.getNamesFromId(entityId);
        const label = getLabel(entityId, enumId);
        return Object.assign(Object.assign({}, this.getTargetAndPath(names, enumId)), { body: `
import React from 'react';
import {
  Dropdown,
  DropdownProps,
} from 'semantic-ui-react';

import { ${enumId}Label } from 'src/services/sonamu.generated';

export function ${enumId}Dropdown(props: DropdownProps) {
  const options = Object.entries(${enumId}Label).map(([key, label]) => {
    return {
      key,
      value: key,
      text: "${label}: " + label,
    };
  });
  return (
    <Dropdown
      className="label"
      options={options}
      {...props}
    />
  );
}
      `.trim(), importKeys: [] });
    }
}
exports.Template__view_enums_dropdown = Template__view_enums_dropdown;
function getLabel(entityId, enumId) {
    if (enumId.endsWith("OrderBy")) {
        return "정렬";
    }
    else if (enumId.endsWith("SearchField")) {
        return "검색";
    }
    else {
        const enumProp = entity_manager_1.EntityManager.get(entityId).props.find((prop) => `${entityId}${(0, inflection_1.camelize)(prop.name)}` === enumId);
        if (enumProp && enumProp.desc) {
            return enumProp.desc;
        }
        return enumId;
    }
}
exports.getLabel = getLabel;
//# sourceMappingURL=view_enums_dropdown.template.js.map