"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Template__view_search_input = void 0;
const entity_manager_1 = require("../entity/entity-manager");
const base_template_1 = require("./base-template");
class Template__view_search_input extends base_template_1.Template {
    constructor() {
        super("view_search_input");
    }
    getTargetAndPath(names) {
        return {
            target: "web/src/components",
            path: `${names.fs}/${names.capital}SearchInput.tsx`,
        };
    }
    render({ entityId }) {
        const names = entity_manager_1.EntityManager.getNamesFromId(entityId);
        return Object.assign(Object.assign({}, this.getTargetAndPath(names)), { body: `
import React from "react";
import { useState } from "react";
import { DropdownProps, Input, InputProps } from "semantic-ui-react";
import { ${names.capital}SearchFieldDropdown } from "src/components/${names.fs}/${names.capital}SearchFieldDropdown";

export function ${names.capital}SearchInput({
  input: { value: inputValue, onChange: inputOnChange, ...inputProps },
  dropdown: dropdownProps,
}: {
  input: InputProps;
  dropdown: DropdownProps;
}) {
  const [keyword, setKeyword] = useState<string>(inputValue ?? '');

  const handleKeyDown = (e: { key: string }) => {
    if (inputOnChange && e.key === 'Enter') {
      inputOnChange(e as any, {
        value: keyword,
      });
    }
  };

  return (
    <Input
      size="small"
      placeholder="검색..."
      style={{ margin: 0 }}
      label={<${names.capital}SearchFieldDropdown {...dropdownProps} />}
      labelPosition="left"
      action={{
        icon: 'search',
        onClick: () => handleKeyDown({ key: 'Enter' }),
      }}
      {...inputProps}
      value={keyword}
      onChange={(e, { value }) => setKeyword(value)}
      onKeyDown={handleKeyDown}
    />
  );
}
      `.trim(), importKeys: [] });
    }
}
exports.Template__view_search_input = Template__view_search_input;
//# sourceMappingURL=view_search_input.template.js.map