"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FixtureManager = exports.FixtureManagerClass = void 0;
const chalk_1 = __importDefault(require("chalk"));
const knex_1 = __importDefault(require("knex"));
const lodash_1 = require("lodash");
const api_1 = require("../api");
const base_model_1 = require("../database/base-model");
const entity_manager_1 = require("../entity/entity-manager");
const types_1 = require("../types/types");
class FixtureManagerClass {
    constructor() {
        this._tdb = null;
        this._fdb = null;
    }
    set tdb(tdb) {
        this._tdb = tdb;
    }
    get tdb() {
        if (this._tdb === null) {
            throw new Error("FixtureManager has not been initialized");
        }
        return this._tdb;
    }
    set fdb(fdb) {
        this._fdb = fdb;
    }
    get fdb() {
        if (this._fdb === null) {
            throw new Error("FixtureManager has not been initialized");
        }
        return this._fdb;
    }
    init() {
        var _a, _b, _c, _d;
        if (this._tdb !== null) {
            return;
        }
        if (api_1.Sonamu.dbConfig.test && api_1.Sonamu.dbConfig.production_master) {
            const tConn = api_1.Sonamu.dbConfig.test.connection;
            const pConn = api_1.Sonamu.dbConfig.production_master
                .connection;
            if (`${(_a = tConn.host) !== null && _a !== void 0 ? _a : "localhost"}:${(_b = tConn.port) !== null && _b !== void 0 ? _b : 3306}/${tConn.database}` ===
                `${(_c = pConn.host) !== null && _c !== void 0 ? _c : "localhost"}:${(_d = pConn.port) !== null && _d !== void 0 ? _d : 3306}/${pConn.database}`) {
                throw new Error(`테스트DB와 프로덕션DB에 동일한 데이터베이스가 사용되었습니다.`);
            }
        }
        this.tdb = (0, knex_1.default)(api_1.Sonamu.dbConfig.test);
        this.fdb = (0, knex_1.default)(api_1.Sonamu.dbConfig.fixture_local);
    }
    cleanAndSeed(usingTables) {
        var _a, e_1, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const tableNames = yield (() => __awaiter(this, void 0, void 0, function* () {
                if (usingTables) {
                    return usingTables;
                }
                const [tables] = yield this.tdb.raw("SHOW TABLE STATUS WHERE Engine IS NOT NULL");
                return tables.map((tableInfo) => tableInfo["Name"]);
            }))();
            yield this.tdb.raw(`SET FOREIGN_KEY_CHECKS = 0`);
            try {
                for (var _d = true, tableNames_1 = __asyncValues(tableNames), tableNames_1_1; tableNames_1_1 = yield tableNames_1.next(), _a = tableNames_1_1.done, !_a; _d = true) {
                    _c = tableNames_1_1.value;
                    _d = false;
                    let tableName = _c;
                    if (tableName == "migrations") {
                        continue;
                    }
                    const [[fdbChecksumRow]] = yield this.fdb.raw(`CHECKSUM TABLE ${tableName}`);
                    const fdbChecksum = fdbChecksumRow["Checksum"];
                    const [[tdbChecksumRow]] = yield this.tdb.raw(`CHECKSUM TABLE ${tableName}`);
                    const tdbChecksum = tdbChecksumRow["Checksum"];
                    if (fdbChecksum !== tdbChecksum) {
                        yield this.tdb(tableName).truncate();
                        const rawQuery = `INSERT INTO ${api_1.Sonamu.dbConfig.test.connection.database}.${tableName}
            SELECT * FROM ${api_1.Sonamu.dbConfig.fixture_local
                            .connection.database}.${tableName}`;
                        yield this.tdb.raw(rawQuery);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_d && !_a && (_b = tableNames_1.return)) yield _b.call(tableNames_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            yield this.tdb.raw(`SET FOREIGN_KEY_CHECKS = 1`);
        });
    }
    getChecksum(db, tableName) {
        return __awaiter(this, void 0, void 0, function* () {
            const [[checksumRow]] = yield db.raw(`CHECKSUM TABLE ${tableName}`);
            return checksumRow.Checksum;
        });
    }
    sync() {
        return __awaiter(this, void 0, void 0, function* () {
            const frdb = (0, knex_1.default)(api_1.Sonamu.dbConfig.fixture_remote);
            const [tables] = yield this.fdb.raw("SHOW TABLE STATUS WHERE Engine IS NOT NULL");
            const tableNames = tables.map((table) => table.Name);
            console.log(chalk_1.default.magenta("SYNC..."));
            yield Promise.all(tableNames.map((tableName) => __awaiter(this, void 0, void 0, function* () {
                if (tableName.startsWith("knex_migrations")) {
                    return;
                }
                const remoteChecksum = yield this.getChecksum(frdb, tableName);
                const localChecksum = yield this.getChecksum(this.fdb, tableName);
                if (remoteChecksum !== localChecksum) {
                    yield this.fdb.transaction((transaction) => __awaiter(this, void 0, void 0, function* () {
                        yield transaction.raw(`SET FOREIGN_KEY_CHECKS = 0`);
                        yield transaction(tableName).truncate();
                        const rows = yield frdb(tableName);
                        console.log(chalk_1.default.blue(tableName), rows.length);
                        yield transaction
                            .insert(rows.map((row) => {
                            Object.keys(row).map((key) => {
                                if (Array.isArray(row[key])) {
                                    row[key] = JSON.stringify(row[key]);
                                }
                            });
                            return row;
                        }))
                            .into(tableName);
                        console.log("OK");
                        yield transaction.raw(`SET FOREIGN_KEY_CHECKS = 0`);
                    }));
                }
            })));
            console.log(chalk_1.default.magenta("DONE!"));
            yield frdb.destroy();
        });
    }
    importFixture(entityId, ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const queries = (0, lodash_1.uniq)((yield Promise.all(ids.map((id) => __awaiter(this, void 0, void 0, function* () {
                return yield this.getImportQueries(entityId, "id", id);
            })))).flat());
            const wdb = base_model_1.BaseModel.getDB("w");
            for (let query of queries) {
                const [rsh] = yield wdb.raw(query);
                console.log({
                    query,
                    info: rsh.info,
                });
            }
        });
    }
    getImportQueries(entityId, field, id) {
        return __awaiter(this, void 0, void 0, function* () {
            console.log({ entityId, field, id });
            const entity = entity_manager_1.EntityManager.get(entityId);
            const wdb = base_model_1.BaseModel.getDB("w");
            // 여기서 실DB의 row 가져옴
            const [row] = yield wdb(entity.table).where(field, id).limit(1);
            if (row === undefined) {
                throw new Error(`${entityId}#${id} row를 찾을 수 없습니다.`);
            }
            // 픽스쳐DB, 실DB
            const fixtureDatabase = api_1.Sonamu.dbConfig.fixture_remote.connection
                .database;
            const realDatabase = api_1.Sonamu.dbConfig.production_master.connection
                .database;
            const selfQuery = `INSERT IGNORE INTO \`${fixtureDatabase}\`.\`${entity.table}\` (SELECT * FROM \`${realDatabase}\`.\`${entity.table}\` WHERE \`id\` = ${id})`;
            const args = Object.entries(entity.relations)
                .filter(([, relation]) => (0, types_1.isBelongsToOneRelationProp)(relation) ||
                ((0, types_1.isOneToOneRelationProp)(relation) &&
                    relation.customJoinClause === undefined))
                .map(([, relation]) => {
                /*
                BelongsToOne인 경우
                  Category / 'id' / row[category_id] 호출
                OneToOne에 joinColumn === true 인 경우
                  Profile / 'id' / row[profile_id] 호출
                OneToOne에 joinColumn === false 인 경우
                  Profile / 'profile_id' / row['id'] 호출
                */
                let field;
                let id;
                if ((0, types_1.isOneToOneRelationProp)(relation) && !relation.hasJoinColumn) {
                    field = `${relation.name}_id`;
                    id = row["id"];
                }
                else {
                    field = "id";
                    id = row[`${relation.name}_id`];
                }
                return {
                    entityId: relation.with,
                    field,
                    id,
                };
            })
                .filter((arg) => arg.id !== null);
            const relQueries = yield Promise.all(args.map((args) => __awaiter(this, void 0, void 0, function* () {
                return this.getImportQueries(args.entityId, args.field, args.id);
            })));
            return [...(0, lodash_1.uniq)(relQueries.reverse().flat()), selfQuery];
        });
    }
    destory() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._tdb) {
                yield this._tdb.destroy();
                this._tdb = null;
            }
            if (this._fdb) {
                yield this._fdb.destroy();
                this._fdb = null;
            }
            yield base_model_1.BaseModel.destroy();
        });
    }
}
exports.FixtureManagerClass = FixtureManagerClass;
exports.FixtureManager = new FixtureManagerClass();
//# sourceMappingURL=fixture-manager.js.map