import { z } from "zod";
export declare type EnumsLabel<T extends string, L extends "ko" | "en"> = {
    [key in T]: {
        [lang in L]: string;
    };
};
export declare type EnumsLabelKo<T extends string> = EnumsLabel<T, "ko">;
export declare const SQLDateTimeString: z.ZodString;
export declare type SQLDateTimeString = z.infer<typeof SQLDateTimeString>;
export declare function zArrayable<T extends z.ZodTypeAny>(shape: T): z.ZodUnion<[T, z.ZodArray<T, "many">]>;
export declare type DistributiveOmit<T, K extends keyof any> = T extends any ? Omit<T, K> : never;
export declare type CommonProp = {
    name: string;
    nullable?: boolean;
    toFilter?: true;
    desc?: string;
    dbDefault?: string | number | {
        raw: string;
    };
};
export declare type IntegerProp = CommonProp & {
    type: "integer";
    unsigned?: true;
};
export declare type BigIntegerProp = CommonProp & {
    type: "bigInteger";
    unsigned?: true;
};
export declare type TextProp = CommonProp & {
    type: "text";
    textType: "text" | "mediumtext" | "longtext";
};
export declare type StringProp = CommonProp & {
    type: "string";
    length: number;
};
export declare type EnumProp = CommonProp & {
    type: "enum";
    length: number;
    id: string;
};
export declare type FloatProp = CommonProp & {
    type: "float";
    unsigned?: true;
    precision: number;
    scale: number;
};
export declare type DoubleProp = CommonProp & {
    type: "double";
    unsigned?: true;
};
export declare type DecimalProp = CommonProp & {
    type: "decimal";
    unsigned?: true;
    precision: number;
    scale: number;
};
export declare type BooleanProp = CommonProp & {
    type: "boolean";
};
export declare type DateProp = CommonProp & {
    type: "date";
};
export declare type DateTimeProp = CommonProp & {
    type: "dateTime";
};
export declare type TimeProp = CommonProp & {
    type: "time";
};
export declare type TimestampProp = CommonProp & {
    type: "timestamp";
};
export declare type JsonProp = CommonProp & {
    type: "json";
    as: {
        ref: string;
    } | z.ZodTypeAny;
};
export declare type UuidProp = CommonProp & {
    type: "uuid";
};
export declare type VirtualProp = CommonProp & {
    type: "virtual";
    as: {
        ref: string;
    } | z.ZodTypeAny;
};
export declare type RelationType = "HasMany" | "BelongsToOne" | "ManyToMany" | "OneToOne";
export declare type RelationOn = "UPDATE" | "CASCADE" | "SET NULL" | "NO ACTION" | "SET DEFAULT" | "RESTRICT";
declare type _RelationProp = {
    type: "relation";
    name: string;
    with: string;
    nullable?: boolean;
    toFilter?: true;
    desc?: string;
};
export declare type OneToOneRelationProp = _RelationProp & {
    relationType: "OneToOne";
    customJoinClause?: string;
} & ({
    hasJoinColumn: true;
    onUpdate: RelationOn;
    onDelete: RelationOn;
} | {
    hasJoinColumn: false;
});
export declare type BelongsToOneRelationProp = _RelationProp & {
    relationType: "BelongsToOne";
    customJoinClause?: string;
    onUpdate: RelationOn;
    onDelete: RelationOn;
};
export declare type HasManyRelationProp = _RelationProp & {
    relationType: "HasMany";
    joinColumn: string;
};
export declare type ManyToManyRelationProp = _RelationProp & {
    relationType: "ManyToMany";
    joinTable: `${string}__${string}`;
    onUpdate: RelationOn;
    onDelete: RelationOn;
};
export declare type RelationProp = OneToOneRelationProp | BelongsToOneRelationProp | HasManyRelationProp | ManyToManyRelationProp;
export declare type EntityProp = IntegerProp | BigIntegerProp | TextProp | StringProp | FloatProp | DoubleProp | DecimalProp | BooleanProp | DateProp | DateTimeProp | TimeProp | TimestampProp | JsonProp | UuidProp | EnumProp | VirtualProp | RelationProp;
export declare type EntityIndex = {
    type: "index" | "unique";
    columns: string[];
    name?: string;
};
export declare type SMDInput<T extends string> = {
    id: string;
    parentId?: string;
    table?: string;
    title?: string;
    props?: EntityProp[];
    indexes?: EntityIndex[];
    subsets?: {
        [subset: string]: T[];
    };
};
export declare type EntityPropNode = {
    nodeType: "plain";
    prop: EntityProp;
} | {
    nodeType: "object" | "array";
    prop?: EntityProp;
    children: EntityPropNode[];
};
export declare function isIntegerProp(p: any): p is IntegerProp;
export declare function isBigIntegerProp(p: any): p is BigIntegerProp;
export declare function isTextProp(p: any): p is TextProp;
export declare function isStringProp(p: any): p is StringProp;
export declare function isEnumProp(p: any): p is EnumProp;
export declare function isFloatProp(p: any): p is FloatProp;
export declare function isDoubleProp(p: any): p is DoubleProp;
export declare function isDecimalProp(p: any): p is DecimalProp;
export declare function isBooleanProp(p: any): p is BooleanProp;
export declare function isDateProp(p: any): p is DateProp;
export declare function isDateTimeProp(p: any): p is DateTimeProp;
export declare function isTimeProp(p: any): p is TimeProp;
export declare function isTimestampProp(p: any): p is TimestampProp;
export declare function isJsonProp(p: any): p is JsonProp;
export declare function isUuidProp(p: any): p is UuidProp;
export declare function isVirtualProp(p: any): p is VirtualProp;
export declare function isRelationProp(p: any): p is RelationProp;
export declare function isOneToOneRelationProp(p: any): p is OneToOneRelationProp;
export declare function isBelongsToOneRelationProp(p: any): p is BelongsToOneRelationProp;
export declare function isHasManyRelationProp(p: any): p is HasManyRelationProp;
export declare function isManyToManyRelationProp(p: any): p is ManyToManyRelationProp;
declare type JoinClause = {
    from: string;
    to: string;
} | {
    custom: string;
};
export declare function isCustomJoinClause(p: any): p is {
    custom: string;
};
declare type SubsetLoader = {
    as: string;
    table: string;
    manyJoin: {
        fromTable: string;
        fromCol: string;
        idField: string;
        toTable: string;
        toCol: string;
        through?: {
            table: string;
            fromCol: string;
            toCol: string;
        };
    };
    oneJoins: ({
        as: string;
        join: "inner" | "outer";
        table: string;
    } & JoinClause)[];
    select: string[];
    loaders?: SubsetLoader[];
};
export declare type SubsetQuery = {
    select: string[];
    virtual: string[];
    joins: ({
        as: string;
        join: "inner" | "outer";
        table: string;
    } & JoinClause)[];
    loaders: SubsetLoader[];
};
export declare type KnexError = {
    code: string;
    errno: number;
    sql: string;
    sqlMessage: string;
    sqlState: string;
};
export declare function isKnexError(e: any): e is KnexError;
export declare type KnexColumnType = "string" | "text" | "smalltext" | "mediumtext" | "longtext" | "integer" | "bigInteger" | "decimal" | "timestamp" | "boolean" | "foreign" | "uuid" | "json" | "float" | "date" | "time" | "dateTime";
export declare type MigrationColumn = {
    name: string;
    type: KnexColumnType;
    nullable: boolean;
    unsigned?: boolean;
    length?: number;
    defaultTo?: string | number;
    precision?: number;
    scale?: number;
};
export declare type MigrationIndex = {
    columns: string[];
    type: "unique" | "index";
};
export declare type MigrationForeign = {
    columns: string[];
    to: string;
    onUpdate: RelationOn;
    onDelete: RelationOn;
};
export declare type MigrationJoinTable = {
    table: string;
    indexes: MigrationIndex[];
    columns: MigrationColumn[];
    foreigns: MigrationForeign[];
};
export declare type MigrationSet = {
    table: string;
    columns: MigrationColumn[];
    indexes: MigrationIndex[];
    foreigns: MigrationForeign[];
};
export declare type MigrationSetAndJoinTable = MigrationSet & {
    joinTables: MigrationJoinTable[];
};
export declare type GenMigrationCode = {
    title: string;
    table: string;
    type: "normal" | "foreign";
    formatted: string | null;
};
export declare type ApiParam = {
    name: string;
    type: ApiParamType;
    optional: boolean;
    defaultDef?: string;
};
export declare namespace ApiParamType {
    type Object = {
        t: "object";
        props: ApiParam[];
    };
    type Union = {
        t: "union";
        types: ApiParamType[];
    };
    type Intersection = {
        t: "intersection";
        types: ApiParamType[];
    };
    type StringLiteral = {
        t: "string-literal";
        value: string;
    };
    type NumericLiteral = {
        t: "numeric-literal";
        value: number;
    };
    type Array = {
        t: "array";
        elementsType: ApiParamType;
    };
    type Ref = {
        t: "ref";
        id: string;
        args?: ApiParamType[];
    };
    type IndexedAccess = {
        t: "indexed-access";
        object: ApiParamType;
        index: ApiParamType;
    };
    type TupleType = {
        t: "tuple-type";
        elements: ApiParamType[];
    };
    type Pick = Ref & {
        t: "ref";
        id: "Pick";
    };
    type Omit = Ref & {
        t: "ref";
        id: "Omit";
    };
    type Partial = Ref & {
        t: "ref";
        id: "Partial";
    };
    type Promise = Ref & {
        t: "ref";
        id: "Promise";
    };
    type Context = Ref & {
        t: "ref";
        id: "Context";
    };
    type TypeParam = {
        t: "type-param";
        id: string;
        constraint?: ApiParamType;
    };
    function isObject(v: any): v is ApiParamType.Object;
    function isUnion(v: any): v is ApiParamType.Union;
    function isIntersection(v: any): v is ApiParamType.Intersection;
    function isStringLiteral(v: any): v is ApiParamType.StringLiteral;
    function isNumericLiteral(v: any): v is ApiParamType.NumericLiteral;
    function isArray(v: any): v is ApiParamType.Array;
    function isRef(v: any): v is ApiParamType.Ref;
    function isIndexedAccess(v: any): v is ApiParamType.IndexedAccess;
    function isTupleType(v: any): v is ApiParamType.TupleType;
    function isPick(v: any): v is ApiParamType.Pick;
    function isOmit(v: any): v is ApiParamType.Omit;
    function isPartial(v: any): v is ApiParamType.Partial;
    function isPromise(v: any): v is ApiParamType.Promise;
    function isContext(v: any): v is ApiParamType.Context;
    function isTypeParam(v: any): v is ApiParamType.TypeParam;
}
export declare type ApiParamType = "string" | "number" | "boolean" | "null" | "undefined" | "void" | "any" | "unknown" | "true" | "false" | ApiParamType.StringLiteral | ApiParamType.NumericLiteral | ApiParamType.Object | ApiParamType.Union | ApiParamType.Intersection | ApiParamType.Array | ApiParamType.Ref | ApiParamType.IndexedAccess | ApiParamType.TypeParam | ApiParamType.TupleType;
export declare const RenderingNode: z.ZodAny;
export declare type RenderingNode = {
    name: string;
    label: string;
    renderType: "string-plain" | "string-image" | "string-datetime" | "string-date" | "number-plain" | "number-id" | "number-fk_id" | "boolean" | "enums" | "array" | "array-images" | "object" | "object-pick" | "record";
    zodType: z.ZodTypeAny;
    element?: RenderingNode;
    children?: RenderingNode[];
    config?: {
        picked: string;
    };
    optional?: boolean;
    nullable?: boolean;
};
export declare const TemplateOptions: z.ZodObject<{
    entity: z.ZodObject<{
        entityId: z.ZodString;
        title: z.ZodString;
        refCode: z.ZodOptional<z.ZodString>;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        title: string;
        refCode?: string | undefined;
    }, {
        entityId: string;
        title: string;
        refCode?: string | undefined;
    }>;
    init_enums: z.ZodObject<{
        entityId: z.ZodString;
        def: z.ZodOptional<z.ZodRecord<z.ZodString, z.ZodRecord<z.ZodString, z.ZodString>>>;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        def?: Record<string, Record<string, string>> | undefined;
    }, {
        entityId: string;
        def?: Record<string, Record<string, string>> | undefined;
    }>;
    init_types: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    init_generated: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    generated: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    generated_http: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    model: z.ZodObject<{
        entityId: z.ZodString;
        defaultSearchField: z.ZodString;
        defaultOrderBy: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        defaultSearchField: string;
        defaultOrderBy: string;
    }, {
        entityId: string;
        defaultSearchField: string;
        defaultOrderBy: string;
    }>;
    model_test: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    bridge: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    service: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    view_list: z.ZodObject<{
        entityId: z.ZodString;
        extra: z.ZodUnknown;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        extra?: unknown;
    }, {
        entityId: string;
        extra?: unknown;
    }>;
    view_list_columns: z.ZodObject<{
        entityId: z.ZodString;
        columns: z.ZodArray<z.ZodObject<{
            name: z.ZodString;
            label: z.ZodString;
            tc: z.ZodString;
        }, "strip", z.ZodTypeAny, {
            name: string;
            label: string;
            tc: string;
        }, {
            name: string;
            label: string;
            tc: string;
        }>, "many">;
        columnImports: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        columns: {
            name: string;
            label: string;
            tc: string;
        }[];
        columnImports: string;
    }, {
        entityId: string;
        columns: {
            name: string;
            label: string;
            tc: string;
        }[];
        columnImports: string;
    }>;
    view_search_input: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    view_form: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    view_id_all_select: z.ZodObject<{
        entityId: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
    }, {
        entityId: string;
    }>;
    view_id_async_select: z.ZodObject<{
        entityId: z.ZodString;
        textField: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        textField: string;
    }, {
        entityId: string;
        textField: string;
    }>;
    view_enums_select: z.ZodObject<{
        entityId: z.ZodString;
        enumId: z.ZodString;
        idConstant: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }>;
    view_enums_dropdown: z.ZodObject<{
        entityId: z.ZodString;
        enumId: z.ZodString;
        idConstant: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }>;
    view_enums_buttonset: z.ZodObject<{
        entityId: z.ZodString;
        enumId: z.ZodString;
        idConstant: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }, {
        entityId: string;
        enumId: string;
        idConstant: string;
    }>;
}, "strip", z.ZodTypeAny, {
    entity: {
        entityId: string;
        title: string;
        refCode?: string | undefined;
    };
    init_enums: {
        entityId: string;
        def?: Record<string, Record<string, string>> | undefined;
    };
    init_types: {
        entityId: string;
    };
    init_generated: {
        entityId: string;
    };
    generated: {
        entityId: string;
    };
    generated_http: {
        entityId: string;
    };
    model: {
        entityId: string;
        defaultSearchField: string;
        defaultOrderBy: string;
    };
    model_test: {
        entityId: string;
    };
    bridge: {
        entityId: string;
    };
    service: {
        entityId: string;
    };
    view_list: {
        entityId: string;
        extra?: unknown;
    };
    view_list_columns: {
        entityId: string;
        columns: {
            name: string;
            label: string;
            tc: string;
        }[];
        columnImports: string;
    };
    view_search_input: {
        entityId: string;
    };
    view_form: {
        entityId: string;
    };
    view_id_all_select: {
        entityId: string;
    };
    view_id_async_select: {
        entityId: string;
        textField: string;
    };
    view_enums_select: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
    view_enums_dropdown: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
    view_enums_buttonset: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
}, {
    entity: {
        entityId: string;
        title: string;
        refCode?: string | undefined;
    };
    init_enums: {
        entityId: string;
        def?: Record<string, Record<string, string>> | undefined;
    };
    init_types: {
        entityId: string;
    };
    init_generated: {
        entityId: string;
    };
    generated: {
        entityId: string;
    };
    generated_http: {
        entityId: string;
    };
    model: {
        entityId: string;
        defaultSearchField: string;
        defaultOrderBy: string;
    };
    model_test: {
        entityId: string;
    };
    bridge: {
        entityId: string;
    };
    service: {
        entityId: string;
    };
    view_list: {
        entityId: string;
        extra?: unknown;
    };
    view_list_columns: {
        entityId: string;
        columns: {
            name: string;
            label: string;
            tc: string;
        }[];
        columnImports: string;
    };
    view_search_input: {
        entityId: string;
    };
    view_form: {
        entityId: string;
    };
    view_id_all_select: {
        entityId: string;
    };
    view_id_async_select: {
        entityId: string;
        textField: string;
    };
    view_enums_select: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
    view_enums_dropdown: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
    view_enums_buttonset: {
        entityId: string;
        enumId: string;
        idConstant: string;
    };
}>;
export declare type TemplateOptions = z.infer<typeof TemplateOptions>;
export declare const TemplateKey: z.ZodEnum<["entity", "init_enums", "init_types", "init_generated", "generated", "generated_http", "model", "model_test", "bridge", "service", "view_list", "view_list_columns", "view_search_input", "view_form", "view_id_all_select", "view_id_async_select", "view_enums_select", "view_enums_dropdown", "view_enums_buttonset"]>;
export declare type TemplateKey = z.infer<typeof TemplateKey>;
export declare const GenerateOptions: z.ZodObject<{
    overwrite: z.ZodOptional<z.ZodBoolean>;
}, "strip", z.ZodTypeAny, {
    overwrite?: boolean | undefined;
}, {
    overwrite?: boolean | undefined;
}>;
export declare type GenerateOptions = z.infer<typeof GenerateOptions>;
export declare const PathAndCode: z.ZodObject<{
    path: z.ZodString;
    code: z.ZodString;
}, "strip", z.ZodTypeAny, {
    path: string;
    code: string;
}, {
    path: string;
    code: string;
}>;
export declare type PathAndCode = z.infer<typeof PathAndCode>;
export {};
//# sourceMappingURL=smd.types.d.ts.map