"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathAndCode = exports.GenerateOptions = exports.TemplateKey = exports.TemplateOptions = exports.RenderingNode = exports.ApiParamType = exports.isKnexError = exports.SonamuQueryMode = exports.isCustomJoinClause = exports.isManyToManyRelationProp = exports.isHasManyRelationProp = exports.isBelongsToOneRelationProp = exports.isOneToOneRelationProp = exports.isRelationProp = exports.isVirtualProp = exports.isUuidProp = exports.isJsonProp = exports.isTimestampProp = exports.isTimeProp = exports.isDateTimeProp = exports.isDateProp = exports.isBooleanProp = exports.isDecimalProp = exports.isDoubleProp = exports.isFloatProp = exports.isEnumProp = exports.isStringProp = exports.isTextProp = exports.isBigIntegerProp = exports.isIntegerProp = exports.zArrayable = exports.SQLDateTimeString = void 0;
const zod_1 = require("zod");
/*
  Custom Scalars
*/
exports.SQLDateTimeString = zod_1.z
    .string()
    .regex(/([0-9]{4}-[0-9]{2}-[0-9]{2}( [0-9]{2}:[0-9]{2}:[0-9]{2})*)$/, {
    message: "잘못된 SQLDate 타입",
})
    .min(10)
    .max(19)
    .describe("SQLDateTimeString");
/*
  Utility Types
*/
function zArrayable(shape) {
    return zod_1.z.union([shape, shape.array()]);
}
exports.zArrayable = zArrayable;
/*
  Prop Type Guards
*/
function isIntegerProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "integer";
}
exports.isIntegerProp = isIntegerProp;
function isBigIntegerProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "bigInteger";
}
exports.isBigIntegerProp = isBigIntegerProp;
function isTextProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "text";
}
exports.isTextProp = isTextProp;
function isStringProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "string";
}
exports.isStringProp = isStringProp;
function isEnumProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "enum";
}
exports.isEnumProp = isEnumProp;
function isFloatProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "float";
}
exports.isFloatProp = isFloatProp;
function isDoubleProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "double";
}
exports.isDoubleProp = isDoubleProp;
function isDecimalProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "decimal";
}
exports.isDecimalProp = isDecimalProp;
function isBooleanProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "boolean";
}
exports.isBooleanProp = isBooleanProp;
function isDateProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "date";
}
exports.isDateProp = isDateProp;
function isDateTimeProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "dateTime";
}
exports.isDateTimeProp = isDateTimeProp;
function isTimeProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "time";
}
exports.isTimeProp = isTimeProp;
function isTimestampProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "timestamp";
}
exports.isTimestampProp = isTimestampProp;
function isJsonProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "json";
}
exports.isJsonProp = isJsonProp;
function isUuidProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "uuid";
}
exports.isUuidProp = isUuidProp;
function isVirtualProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "virtual";
}
exports.isVirtualProp = isVirtualProp;
function isRelationProp(p) {
    return (p === null || p === void 0 ? void 0 : p.type) === "relation";
}
exports.isRelationProp = isRelationProp;
function isOneToOneRelationProp(p) {
    return (p === null || p === void 0 ? void 0 : p.relationType) === "OneToOne";
}
exports.isOneToOneRelationProp = isOneToOneRelationProp;
function isBelongsToOneRelationProp(p) {
    return (p === null || p === void 0 ? void 0 : p.relationType) === "BelongsToOne";
}
exports.isBelongsToOneRelationProp = isBelongsToOneRelationProp;
function isHasManyRelationProp(p) {
    return (p === null || p === void 0 ? void 0 : p.relationType) === "HasMany";
}
exports.isHasManyRelationProp = isHasManyRelationProp;
function isManyToManyRelationProp(p) {
    return (p === null || p === void 0 ? void 0 : p.relationType) === "ManyToMany";
}
exports.isManyToManyRelationProp = isManyToManyRelationProp;
function isCustomJoinClause(p) {
    return p === null || p === void 0 ? void 0 : p.custom;
}
exports.isCustomJoinClause = isCustomJoinClause;
/* BaseModel */
exports.SonamuQueryMode = zod_1.z.enum(["both", "list", "count"]);
function isKnexError(e) {
    return e.code && e.sqlMessage && e.sqlState;
}
exports.isKnexError = isKnexError;
var ApiParamType;
(function (ApiParamType) {
    function isObject(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "object";
    }
    ApiParamType.isObject = isObject;
    function isUnion(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "union";
    }
    ApiParamType.isUnion = isUnion;
    function isIntersection(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "intersection";
    }
    ApiParamType.isIntersection = isIntersection;
    function isStringLiteral(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "string-literal";
    }
    ApiParamType.isStringLiteral = isStringLiteral;
    function isNumericLiteral(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "numeric-literal";
    }
    ApiParamType.isNumericLiteral = isNumericLiteral;
    function isArray(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "array";
    }
    ApiParamType.isArray = isArray;
    function isRef(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref";
    }
    ApiParamType.isRef = isRef;
    function isIndexedAccess(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "indexed-access";
    }
    ApiParamType.isIndexedAccess = isIndexedAccess;
    function isTupleType(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "tuple-type";
    }
    ApiParamType.isTupleType = isTupleType;
    function isPick(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Pick";
    }
    ApiParamType.isPick = isPick;
    function isOmit(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Omit";
    }
    ApiParamType.isOmit = isOmit;
    function isPartial(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Partial";
    }
    ApiParamType.isPartial = isPartial;
    function isPromise(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Promise";
    }
    ApiParamType.isPromise = isPromise;
    function isContext(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Context";
    }
    ApiParamType.isContext = isContext;
    function isRefKnex(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "ref" && v.id === "Knex";
    }
    ApiParamType.isRefKnex = isRefKnex;
    function isTypeParam(v) {
        return (v === null || v === void 0 ? void 0 : v.t) === "type-param";
    }
    ApiParamType.isTypeParam = isTypeParam;
})(ApiParamType || (exports.ApiParamType = ApiParamType = {}));
/* Template */
// 셀프 참조 타입이므로 Zod 생략하고 직접 정의
exports.RenderingNode = zod_1.z.any();
exports.TemplateOptions = zod_1.z.object({
    entity: zod_1.z.object({
        entityId: zod_1.z.string(),
        parentId: zod_1.z.string().optional(),
        title: zod_1.z.string(),
        table: zod_1.z.string().optional(),
    }),
    init_types: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    generated: zod_1.z.object({}),
    generated_sso: zod_1.z.object({}),
    generated_http: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    model: zod_1.z.object({
        entityId: zod_1.z.string(),
        defaultSearchField: zod_1.z.string(),
        defaultOrderBy: zod_1.z.string(),
    }),
    model_test: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    bridge: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    service: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    view_list: zod_1.z.object({
        entityId: zod_1.z.string(),
        extra: zod_1.z.unknown(),
    }),
    view_list_columns: zod_1.z.object({
        entityId: zod_1.z.string(),
        columns: zod_1.z
            .object({
            name: zod_1.z.string(),
            label: zod_1.z.string(),
            tc: zod_1.z.string(),
        })
            .array(),
        columnImports: zod_1.z.string(),
    }),
    view_search_input: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    view_form: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    view_id_all_select: zod_1.z.object({
        entityId: zod_1.z.string(),
    }),
    view_id_async_select: zod_1.z.object({
        entityId: zod_1.z.string(),
        textField: zod_1.z.string(),
    }),
    view_enums_select: zod_1.z.object({
        entityId: zod_1.z.string(),
        enumId: zod_1.z.string(),
    }),
    view_enums_dropdown: zod_1.z.object({
        entityId: zod_1.z.string(),
        enumId: zod_1.z.string(),
    }),
    view_enums_buttonset: zod_1.z.object({
        entityId: zod_1.z.string(),
        enumId: zod_1.z.string(),
    }),
});
exports.TemplateKey = zod_1.z.enum([
    "entity",
    "init_types",
    "generated",
    "generated_sso",
    "generated_http",
    "model",
    "model_test",
    "bridge",
    "service",
    "view_list",
    "view_list_columns",
    "view_search_input",
    "view_form",
    "view_id_all_select",
    "view_id_async_select",
    "view_enums_select",
    "view_enums_dropdown",
    "view_enums_buttonset",
]);
exports.GenerateOptions = zod_1.z.object({
    overwrite: zod_1.z.boolean().optional(),
});
exports.PathAndCode = zod_1.z.object({
    path: zod_1.z.string(),
    code: zod_1.z.string(),
});
//# sourceMappingURL=types.js.map